//
//  UITableView+Extensions.swift
//  SpokestackTray
//
//  Created by Cory D. Wiles on 8/11/20.
//

import Foundation
import UIKit

extension UITableView {
    
    // MARK: Internal (methods)
    
    func spsk_scrollToBottomRow() -> Void {
        
        DispatchQueue.main.async {
        
            guard self.numberOfSections > 0 else {
                return
            }

            /// Make an attempt to use the bottom-most section with at least one row
            
            var section: Int = max(self.numberOfSections - 1, 0)
            var row: Int = max(self.numberOfRows(inSection: section) - 1, 0)
            var indexPath: IndexPath = IndexPath(row: row, section: section)

            /// Ensure the index path is valid, otherwise use the section above (sections can
            /// contain 0 rows which leads to an invalid index path)
            
            while !self.indexPathIsValid(indexPath) {
            
                section = max(section - 1, 0)
                row = max(self.numberOfRows(inSection: section) - 1, 0)
                indexPath = IndexPath(row: row, section: section)

                /// If we're down to the last section, attempt to use the first row
                
                if indexPath.section == 0 {
                
                    indexPath = IndexPath(row: 0, section: 0)
                    break
                }
            }

            /// In the case that [0, 0] is valid (perhaps no data source?), ensure we don't encounter an
            /// exception here
            
            guard self.indexPathIsValid(indexPath) else {
                return
            }

            self.scrollToRow(at: indexPath, at: .bottom, animated: true)
        }
    }
    
    // MARK: Private (methods)

    private func indexPathIsValid(_ indexPath: IndexPath) -> Bool {
        
        let section: Int = indexPath.section
        let row: Int = indexPath.row
        
        return section < self.numberOfSections && row < self.numberOfRows(inSection: section)
    }
}
